# lingo - basket and payment system
# Copyright (C) 2015  Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.conf.urls import re_path
from django.urls import include

from combo.urls_utils import decorated_includes, staff_required

from .manager_views import (
    BasketItemErrorListView,
    BasketItemMarkAsNotifiedView,
    PaymentBackendCreateView,
    PaymentBackendDeleteView,
    PaymentBackendListView,
    PaymentBackendUpdateView,
    RegieCreateView,
    RegieDeleteView,
    RegieListView,
    RegieUpdateView,
    TransactionListView,
    download_transactions_csv,
)
from .views import (
    AddBasketItemApiView,
    BasketItemPayView,
    CallbackView,
    CancelItemView,
    CancelTransactionApiView,
    CreditDownloadView,
    ItemDownloadView,
    ItemPaymentsDownloadView,
    ItemView,
    PaymentDownloadView,
    PaymentStatusView,
    PayView,
    RegiesApiView,
    RemoveBasketItemApiView,
    ReturnView,
    SelfInvoiceView,
    TransactionStatusApiView,
    ValidateTransactionApiView,
)

lingo_manager_urls = [
    re_path('^$', TransactionListView.as_view(), name='lingo-manager-homepage'),
    re_path('^payments/error/$', BasketItemErrorListView.as_view(), name='lingo-manager-payment-error-list'),
    re_path(
        r'^item/(?P<item_id>\d+)/mark-as-notified/$',
        BasketItemMarkAsNotifiedView.as_view(),
        name='lingo-manager-basket-item-mark-as-notified',
    ),
    re_path(
        '^transactions/download-csv/$', download_transactions_csv, name='lingo-manager-transactions-download'
    ),
    re_path('^regies/$', RegieListView.as_view(), name='lingo-manager-regie-list'),
    re_path('^regies/add/$', RegieCreateView.as_view(), name='lingo-manager-regie-add'),
    re_path(r'^regies/(?P<pk>\d+)/edit$', RegieUpdateView.as_view(), name='lingo-manager-regie-edit'),
    re_path(r'^regies/(?P<pk>\d+)/delete$', RegieDeleteView.as_view(), name='lingo-manager-regie-delete'),
    re_path('^paymentbackends/$', PaymentBackendListView.as_view(), name='lingo-manager-paymentbackend-list'),
    re_path(
        '^paymentbackends/add/$', PaymentBackendCreateView.as_view(), name='lingo-manager-paymentbackend-add'
    ),
    re_path(
        r'^paymentbackends/(?P<pk>\d+)/edit$',
        PaymentBackendUpdateView.as_view(),
        name='lingo-manager-paymentbackend-edit',
    ),
    re_path(
        r'^paymentbackends/(?P<pk>\d+)/delete$',
        PaymentBackendDeleteView.as_view(),
        name='lingo-manager-paymentbackend-delete',
    ),
]

urlpatterns = [
    re_path('^api/lingo/regies$', RegiesApiView.as_view(), name='api-regies'),
    re_path('^api/lingo/add-basket-item$', AddBasketItemApiView.as_view(), name='api-add-basket-item'),
    re_path(
        '^api/lingo/remove-basket-item$', RemoveBasketItemApiView.as_view(), name='api-remove-basket-item'
    ),
    re_path(
        '^api/lingo/validate-transaction$',
        ValidateTransactionApiView.as_view(),
        name='api-validate-transaction',
    ),
    re_path(
        '^api/lingo/cancel-transaction$', CancelTransactionApiView.as_view(), name='api-cancel-transaction'
    ),
    re_path(
        '^api/lingo/transaction-status/(?P<transaction_signature>.+)/$',
        TransactionStatusApiView.as_view(),
        name='api-transaction-status',
    ),
    re_path(r'^lingo/pay$', PayView.as_view(), name='lingo-pay'),
    re_path(r'^lingo/cancel/(?P<pk>\d+)/$', CancelItemView.as_view(), name='lingo-cancel-item'),
    re_path(r'^lingo/callback/(?P<regie_pk>\d+)/$', CallbackView.as_view(), name='lingo-callback'),
    re_path(
        r'^lingo/callback-payment-backend/(?P<payment_backend_pk>[A-Za-z0-9_-]+)/$',
        CallbackView.as_view(),
        name='lingo-callback-payment-backend',
    ),
    re_path(r'^lingo/return/(?P<regie_pk>\d+)/$', ReturnView.as_view(), name='lingo-return'),
    re_path(
        r'^lingo/return-payment-backend/(?P<payment_backend_pk>\d+)/(?P<transaction_signature>.+)/$',
        ReturnView.as_view(),
        name='lingo-return-payment-backend',
    ),
    re_path(r'^manage/lingo/', decorated_includes(staff_required, include(lingo_manager_urls))),
    re_path(
        r'^lingo/item/(?P<regie_id>[\w,-]+)/(?P<item_crypto_id>[\w,-]+)/(?P<cell_crypto_reference>[\w,-]+)/pdf$',
        ItemDownloadView.as_view(),
        name='download-item-pdf',
    ),
    re_path(
        r'^lingo/item/(?P<regie_id>[\w,-]+)/(?P<item_crypto_id>[\w,-]+)/(?P<cell_crypto_reference>[\w,-]+)/payments/pdf$',
        ItemPaymentsDownloadView.as_view(),
        name='download-item-payments-pdf',
    ),
    re_path(
        r'^lingo/item/(?P<regie_id>[\w,-]+)/(?P<item_crypto_id>[\w,-]+)/(?P<cell_crypto_reference>[\w,-]+)/$',
        ItemView.as_view(),
        name='view-item',
    ),
    re_path(
        r'^lingo/payment/(?P<regie_id>[\w,-]+)/(?P<payment_crypto_id>[\w,-]+)/(?P<cell_crypto_reference>[\w,-]+)/pdf$',
        PaymentDownloadView.as_view(),
        name='download-payment-pdf',
    ),
    re_path(
        r'^lingo/credit/(?P<regie_id>[\w,-]+)/(?P<credit_crypto_id>[\w,-]+)/(?P<cell_crypto_reference>[\w,-]+)/pdf$',
        CreditDownloadView.as_view(),
        name='download-credit-pdf',
    ),
    re_path(
        r'^lingo/item/(?P<item_signature>.+)/pay$', BasketItemPayView.as_view(), name='basket-item-pay-view'
    ),
    re_path(r'^lingo/payment-status$', PaymentStatusView.as_view(), name='payment-status'),
    re_path(r'^lingo/self-invoice/(?P<cell_id>\d+)/$', SelfInvoiceView.as_view(), name='lingo-self-invoice'),
]
