# combo - content management system
# Copyright (C) 2017  Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json

from django.http import HttpResponse, HttpResponseForbidden
from django.shortcuts import get_object_or_404
from django.views.generic.base import View

from .models import Map


class GeojsonView(View):
    def get(self, request, *args, **kwargs):
        cell = get_object_or_404(Map, pk=kwargs['cell_id'])
        layer = get_object_or_404(cell.layers.all(), kind='geojson', slug=kwargs['layer_slug'])
        if not cell.page.is_visible(request.user) or not cell.is_visible(request):
            return HttpResponseForbidden()
        options = cell.maplayeroptions_set.get(map_layer=layer)
        geojson = layer.get_geojson(request, options.properties)
        content_type = 'application/json'
        return HttpResponse(json.dumps(geojson), content_type=content_type)
