# combo - content management system
# Copyright (C) 2014  Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import ckeditor.views as ckeditor_views
from django.conf.urls import re_path
from django.urls import path
from django.views.decorators.cache import never_cache
from django.views.decorators.clickjacking import xframe_options_sameorigin

from combo.apps.assets import views as assets_views
from combo.urls_utils import pages_admin_required

from .. import plugins
from . import views

urlpatterns = [
    re_path(r'^$', views.homepage, name='combo-manager-homepage'),
    re_path(r'^menu.json$', views.menu_json),
    re_path(r'^site-export$', pages_admin_required(views.site_export), name='combo-manager-site-export'),
    re_path(r'^site-import$', pages_admin_required(views.site_import), name='combo-manager-site-import'),
    re_path(r'^site-settings$', views.site_settings, name='combo-manager-site-settings'),
    path('models/<model_name>/ajax-choices', views.select2_choices, name='combo-manager-select2-choices'),
    re_path(
        r'^cells/invalid-report/$',
        pages_admin_required(views.invalid_cell_report),
        name='combo-manager-invalid-cell-report',
    ),
    re_path(r'^pages/add/$', views.page_add, name='combo-manager-page-add'),
    re_path(r'^pages/(?P<pk>\d+)/$', views.page_view, name='combo-manager-page-view'),
    re_path(
        r'^pages/(?P<pk>\d+)/template$', views.page_select_template, name='combo-manager-page-select-template'
    ),
    re_path(r'^pages/(?P<pk>\d+)/visibility$', views.page_visibility, name='combo-manager-page-visibility'),
    re_path(
        r'^pages/(?P<pk>\d+)/redirection$',
        views.page_edit_redirection,
        name='combo-manager-page-edit-redirection',
    ),
    re_path(
        r'^pages/(?P<pk>\d+)/include-in-navigation$',
        views.page_edit_include_in_navigation,
        name='combo-manager-page-edit-include-in-navigation',
    ),
    re_path(r'^pages/(?P<pk>\d+)/slug$', views.page_edit_slug, name='combo-manager-page-edit-slug'),
    re_path(
        r'^pages/(?P<pk>\d+)/linked-card$',
        views.page_edit_linked_card,
        name='combo-manager-page-edit-linked-card',
    ),
    re_path(r'^pages/(?P<pk>\d+)/title$', views.page_edit_title, name='combo-manager-page-edit-title'),
    re_path(
        r'^pages/(?P<pk>\d+)/description$',
        views.page_edit_description,
        name='combo-manager-page-edit-description',
    ),
    re_path(r'^pages/(?P<pk>\d+)/picture/$', views.page_edit_picture, name='combo-manager-page-edit-picture'),
    re_path(
        r'^pages/(?P<pk>\d+)/remove-picture/$',
        views.page_remove_picture,
        name='combo-manager-page-remove-picture',
    ),
    re_path(
        r'^pages/(?P<pk>\d+)/extra-variables/$',
        views.page_edit_extra_variables,
        name='combo-manager-page-edit-extra-variables',
    ),
    re_path(
        r'^pages/(?P<pk>\d+)/delete$',
        pages_admin_required(views.page_delete),
        name='combo-manager-page-delete',
    ),
    re_path(r'^pages/(?P<pk>\d+)/export$', views.page_export, name='combo-manager-page-export'),
    re_path(
        r'^pages/(?P<pk>\d+)/add/$',
        pages_admin_required(views.page_add_child),
        name='combo-manager-page-add-child',
    ),
    re_path(
        r'^pages/(?P<pk>\d+)/duplicate$',
        pages_admin_required(views.page_duplicate),
        name='combo-manager-page-duplicate',
    ),
    re_path(
        r'^pages/(?P<pk>\d+)/save',
        pages_admin_required(views.snapshot_save),
        name='combo-manager-page-save',
    ),
    re_path(
        r'^pages/(?P<pk>\d+)/edit-roles/$',
        pages_admin_required(views.page_edit_roles),
        name='combo-manager-page-edit-roles',
    ),
    re_path(r'^pages/(?P<pk>\d+)/inspect/$', views.page_inspect, name='combo-manager-page-inspect'),
    re_path(r'^pages/(?P<pk>\d+)/history$', views.page_history, name='combo-manager-page-history'),
    re_path(
        r'^pages/(?P<page_pk>\d+)/history/(?P<pk>\d+)/$',
        pages_admin_required(views.snapshot_restore),
        name='combo-manager-snapshot-restore',
    ),
    re_path(
        r'^pages/(?P<page_pk>\d+)/history/(?P<pk>\d+)/view/$',
        views.snapshot_view,
        name='combo-manager-snapshot-view',
    ),
    re_path(
        r'^pages/(?P<page_pk>\d+)/history/(?P<pk>\d+)/inspect/$',
        views.snapshot_inspect,
        name='combo-manager-snapshot-inspect',
    ),
    re_path(
        r'^pages/(?P<page_pk>\d+)/history/(?P<pk>\d+)/export$',
        pages_admin_required(views.snapshot_export),
        name='combo-manager-snapshot-export',
    ),
    re_path(
        r'^pages/(?P<pk>\d+)/history/compare/$',
        views.page_history_compare,
        name='combo-manager-page-history-compare',
    ),
    re_path(
        r'^pages/(?P<page_pk>\d+)/add-cell-to-(?P<ph_key>[\w_-]+)/(?P<cell_type>\w+)/(?P<variant>[\w-]+)/$',
        views.page_add_cell,
        name='combo-manager-page-add-cell',
    ),
    re_path(
        r'^pages/(?P<page_pk>\d+)/cell/(?P<cell_reference>[\w_-]+)/$',
        views.page_edit_cell,
        name='combo-manager-page-edit-cell',
    ),
    re_path(
        r'^pages/(?P<page_pk>\d+)/cell/(?P<cell_reference>[\w_-]+)/delete$',
        views.page_delete_cell,
        name='combo-manager-page-delete-cell',
    ),
    re_path(
        r'^pages/(?P<page_pk>\d+)/cell/(?P<cell_reference>[\w_-]+)/duplicate$',
        views.page_duplicate_cell,
        name='combo-manager-page-duplicate-cell',
    ),
    re_path(
        r'^pages/(?P<page_pk>\d+)/cell/(?P<cell_reference>[\w_-]+)/attribute/(?P<attribute>[\w_]+)/$',
        views.page_edit_cell_attribute,
        name='combo-manager-page-edit-cell-attribute',
    ),
    re_path(
        r'^pages/(?P<page_pk>\d+)/cell/(?P<cell_reference>[\w_-]+)/add-link/(?P<link_code>[\w-]+)$',
        views.page_list_cell_add_link,
        name='combo-manager-page-list-cell-add-link',
    ),
    re_path(
        r'^pages/(?P<page_pk>\d+)/cell/(?P<cell_reference>[\w_-]+)/link/(?P<link_cell_reference>[\w_-]+)/$',
        views.page_list_cell_edit_link,
        name='combo-manager-page-list-cell-edit-link',
    ),
    re_path(
        r'^pages/(?P<page_pk>\d+)/cell/(?P<cell_reference>[\w_-]+)/link/(?P<link_cell_reference>[\w_-]+)/delete$',
        views.page_list_cell_delete_link,
        name='combo-manager-page-list-cell-delete-link',
    ),
    re_path(
        r'^pages/(?P<page_pk>\d+)/cell/(?P<cell_reference>[\w_-]+)/order$',
        views.link_list_order,
        name='combo-manager-link-list-order',
    ),
    re_path(r'^pages/(?P<page_pk>\d+)/order$', views.cell_order, name='combo-manager-cell-order'),
    re_path(
        r'^pages/(?P<page_pk>\d+)/placeholder/(?P<placeholder>[\w_-]+)/options$',
        views.placeholder_options,
        name='combo-manage-placeholder-options',
    ),
    re_path(r'^pages/order$', views.page_order, name='combo-manager-page-order'),
    re_path(
        r'^pages/(?P<page_path>[\w/_-]+)/$',
        views.page_redirect_to_edit,
        name='combo-manager-page-redirect-to-edit',
    ),
    re_path(r'^ckeditor/upload/', xframe_options_sameorigin(ckeditor_views.upload), name='ckeditor_upload'),
    re_path(r'^ckeditor/browse/', never_cache(assets_views.browse), name='ckeditor_browse'),
]

urlpatterns = plugins.register_plugins_manager_urls(urlpatterns)
